import pygame
from pygame.locals import *

from pykitten import utils
from pykitten.gui import TextLine, ClickArea


from resources import GFX_wobbly, SFX, load_mapdata, GFX_bg, JB
from mrwobbly import MrWobbly
from basestate import BaseState, FadeState
from map import *
from gfxmap import *

class Game(BaseState):
    def __init__(self, mapdata):
        BaseState.__init__(self)
        if not mapdata:
            self.quit()
            return

        self.fps = 60
        self.sprites = pygame.sprite.Group()


        #timer for pipe movement
        self.timer = utils.Timer(5)

        #map layer
        self.mapdata = mapdata
        self.mapcopy = mapdata.copy()

        self.map_layer = pygame.Surface((800,544))
        self.map_layer.set_colorkey((255,0,255))
        self.pipe_layer = pygame.Surface((800,544))
        self.pipe_layer.set_colorkey((255,0,255))
        self.pipe_layer.fill((255,0,255))

        draw_map(self.mapdata, self.map_layer)
        draw_pipes(self.mapdata, self.pipe_layer)

        #get gfxmap
        self.gfxmap = GfxMap(mapdata)


        #player
        self.player = MrWobbly(GFX_wobbly)
        self.gfxmap.place_player(self.player)
        self.sprites.add(self.player)

        self.old_y = self.player.pos[1]

        #black message box
        self.no_msg = pygame.Surface((800,24))
        self.no_msg.fill((0,0,0))
        self.no_msg_pos=utils.center(self.no_msg.get_size(),(800,600))
        self.no_msg_pos[1] = 600-self.no_msg.get_size()[1]

        #keyboard bindings
        self.add_kbevent(KEYDOWN, K_r, callback=self.reload)
        self.add_kbevent(KEYDOWN, K_F12, callback=self.reload)

        #player movement
        self.add_kbevent(KEYDOWN, K_RIGHT,
            callback=self.player.move, dir=1)
        self.add_kbevent(KEYUP, K_RIGHT,
            callback=self.player.move, dir=0)
        self.add_kbevent(KEYDOWN, K_LEFT,
            callback=self.player.move, dir=-1)
        self.add_kbevent(KEYUP, K_LEFT,
            callback=self.player.move, dir=0)

        self.add_kbevent(KEYDOWN, K_UP, callback=self.use) #use btn
        self.add_kbevent(KEYDOWN, K_DOWN, callback=self.pipe) #use pipe

    def use(self):
        colnr = self.gfxmap.player_on_btn(self.player)
        if colnr is not None:
            self.gfxmap.use_btn(colnr)
            SFX["click.ogg"].play()
            draw_map(self.mapdata, self.map_layer)
            draw_pipes(self.mapdata, self.pipe_layer)

    def pipe(self):
        pipe_key = self.gfxmap.player_on_pipeexit(self.player)
        if not pipe_key:
            return
        self.player.pipe(pipe_key)

    def unpipe(self):
        self.player.unpipe()

    def update_other(self):
        if self.player.mode==0 and not self.player.falling and \
            self.gfxmap.player_on_finish(self.player): #level done
            self.quit()
        if self.player.mode == 1 and self.timer.update(): #piping
            self.gfxmap.pipe_player(self.player)
        elif self.player.mode == 42: # player squished
            self.show_msg("Press <r> to try again.")


    def show_msg(self, txt):
        #show text on bottom of screen
        self.background.blit(self.no_msg, self.no_msg_pos)
        text = TextLine(txt, fontsize=12)
        pos = utils.center(text.get_size(), ((800,600)))
        pos[1] = 590 - text.get_size()[1]
        self.background.blit(text, pos)

    def update_screen(self):
        self.screen.blit(self.background, (0,0))
        self.screen.blit(self.map_layer, (0,32))

        self.sprites.update()
        self.sprites.draw(self.screen)
        self.screen.blit(self.pipe_layer, (0,32))

        self.gfxmap.check_collisions(self.player)
        if self.player.mode==0 and self.player.pos[1] > self.old_y:
            self.player.fall()
        else:
            self.player.unfall()
        self.old_y = self.player.pos[1]
        pygame.display.flip()


    def reload(self):
        mapdata = self.mapcopy.copy()
        self.next = Game(mapdata)
        self.quit()


class EditorGame(Game):
    def __init__(self, mapdata):
        Game.__init__(self, mapdata)

        #get out of here...
        self.add_kbevent(KEYDOWN, K_ESCAPE, callback=self.quit)

    def reload(self):
        mapdata = self.mapcopy.copy()
        self.next = EditorGame(mapdata)
        self.quit()


class CampaignGame(Game):
    def __init__(self, mapdata):
        Game.__init__(self, mapdata)

        if mapdata.filename=="0001":
            self.show_msg("Press LEFT and RIGHT arrow keys to move.")
        elif mapdata.filename=="0002":
            self.show_msg("Press DOWN to use pipes.")
        elif mapdata.filename=="0003":
            self.show_msg("Press UP to use buttons.")

        #get out of here...
        self.add_kbevent(KEYDOWN, K_ESCAPE, callback=self.do_quit)
        self.reloading = False


    def do_quit(self):
        self.kill_prev = True  #kill campaign state
        self.quit()

    def reload(self):
        mapdata = self.mapcopy.copy()
        self.reloading = True
        self.next = CampaignGame(mapdata)
        self.quit()

    def quit(self):
        if self.reloading:
            Game.quit(self)
            return
        if self.mapdata.filename=="0015":
            pygame.mixer.music.fadeout(30)
            self.next = FadeState(self.background, fade=5)
        Game.quit(self)



class CustomGame(EditorGame):
    pass


class CredGame(Game):
    def __init__(self):
        mapdata = load_mapdata("zk", "stuff")
        Game.__init__(self, mapdata)
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)
        self.player.set_pos((320, 352))
        self.player.pipe((10,10))
        self.background.blit(GFX_bg["credits.png"], (0,0))

        r = ClickArea(pygame.rect.Rect([0,0,800,600]),
            callback=self.quit)
        self.widgets.append(r)



