import pygame

from pykitten import data

from resources import GFX_tiles, GFX_mtiles, COLORS

class TileData:
    #pickleable Tiledata
    def __init__(self, type, pos):
        self.type = type
        self.pos = pos #grid coordinates
        self.rot = 0 #rotation of image

        self.block = 1 #blocking part of tjunction
        #1: left blocked, down and right free
        #2: down blocked, left and right free
        #3: right blocked, left and down free

        #color-coded connection between button/pipe
        self.color = None #index nr in COLORS

        #0,1,2,3 - up, left, down, right
        self.set_free()

    def copy(self):
        new = TileData(self.type, self.pos)
        new.type = self.type
        new.pos = self.pos
        new.rot = self.rot
        new.block = self.block
        new.color = self.color
        new.free = self.free
        return new

    def set_free(self):
        if self.type=="pipe.png":
            self.free = [1, 3]
        elif self.type=="pipe_exit.png":
            self.free = [3]
        elif self.type=="pipe_tjunc.png":
            self.free = [1, 2, 3]
        elif self.type=="pipe_turn.png":
            self.free = [1, 2]
        else:
            self.free = []

    def add_rot(self):
        self.rot += 1
        if self.rot > 3:
            self.rot = 0

        new_free = []
        for f in self.free:
            new_f = f + 1
            if new_f > 3:
                new_f = 0
            new_free.append(new_f)
        #print self.free, new_free
        self.free = new_free

    def switch_block(self):
        if self.type == "pipe_tjunc.png":
            self.block += 1
            if self.block > 3:
                self.block = 1



    def switch_type(self, type):
        self.type = type
        if type=="pipe_tjunc.png":
            self.block = 1
        else:
            self.block = None
        self.rot = 0
        self.set_free()

    def switch_color(self, color_nr):
        if self.type=="empty.png" or self.type.endswith("exit"):
            return

        self.color = color_nr




class MapData:
    #pickleable Mapdata
    def __init__(self):
        #generate empty default map (25x17)
        self.tiles = {}
        self.pipes = {}
        width, height = 25, 17
        self.filename = None
        for w in xrange(width):
            for h in xrange(height):
                self.tiles[w,h] = TileData("empty.png", (w,h))
                self.pipes[w,h] = TileData("empty.png", (w,h))


    def copy(self):
        new = MapData()
        new.tiles = self.tiles.copy()
        new.pipes = self.pipes.copy()
        for key in new.tiles.keys():
            new.tiles[key] = new.tiles[key].copy()
            new.pipes[key] = new.pipes[key].copy()
        new.filename = self.filename
        return new


def draw_map(mapdata, surface):
    surface.fill((255,0,255))
    for k in mapdata.tiles.keys():
        #draw tile
        t = mapdata.tiles[k]
        img = GFX_tiles[t.type].copy()

        if t.color is not None and t.type=="btn.png": #colorize button
            col = GFX_tiles["btn_clr.png"].copy()
            col = colorize(col, COLORS[t.color])
            img.blit(col, (0,0))
        surface.blit(img, (t.pos[0]*32, t.pos[1]*32))


def draw_pipes(mapdata, surface):
    #draw pipe
    surface.fill((255,0,255))
    for k in mapdata.tiles.keys():

        p = mapdata.pipes[k]
        img = GFX_tiles[p.type].copy()

        if p.type == "pipe_tjunc.png":
            block = GFX_tiles["pipe_block.png"].copy()
            if p.block==2: #down blocked
                block = pygame.transform.rotate(block, 90)
            elif p.block==3: #right blocked
                block = pygame.transform.rotate(block, 180)

            img.blit(block, (0,0))

        if p.color is not None: #colorize pipe
            col = GFX_tiles["pipe_clr.png"].copy()
            col = colorize(col, COLORS[p.color])
            img.blit(GFX_tiles["pipe_sign.png"], (0,0))
            img.blit(col, (0,0))

        if p.rot:
            img = pygame.transform.rotate(img, p.rot*90)
        surface.blit(img, (p.pos[0]*32, p.pos[1]*32))


def draw_preview(mapdata, surface):
    surface.fill((0,0,0))
    w = 16
    for key in mapdata.tiles.keys():

        #draw mini tiles
        t = mapdata.tiles[key]
        img = GFX_mtiles[t.type].copy()
        if t.color is not None and t.type=="btn.png": #colorize button
            col = GFX_mtiles["btn_clr.png"].copy()
            col = colorize(col, COLORS[t.color])
            img.blit(col, (0,0))
        surface.blit(img, (t.pos[0]*w, t.pos[1]*w))

        #draw mini pipe
        p = mapdata.pipes[key]
        img = GFX_mtiles[p.type].copy()

        if p.type == "pipe_tjunc.png":
            block = GFX_mtiles["pipe_block.png"].copy()
            if p.block==2: #down blocked
                block = pygame.transform.rotate(block, 90)
            elif p.block==3: #right blocked
                block = pygame.transform.rotate(block, 180)

            img.blit(block, (0,0))

        if p.color is not None: #colorize pipe
            col = GFX_mtiles["pipe_clr.png"].copy()
            col = colorize(col, COLORS[p.color])
            img.blit(col, (0,0))

        if p.rot:
            img = pygame.transform.rotate(img, p.rot*90)
        surface.blit(img, (p.pos[0]*w, p.pos[1]*w))


def colorize(surface, color):
    #fill everyting non-transparent with color
    surface = surface.copy()
    w, h = surface.get_size()
    for y in xrange(h):
        for x in xrange(w):
            if surface.get_at((x,y)) != (255,0,255):
                surface.set_at((x,y), color)
    return surface



def save_map(mapdata, filename, folder):
    path = data.filepath(folder, filename)

    try:
        mapdata.filename = filename[:-4]
        data.save_object(mapdata, path)
        return 1
    except:
        return 0
